/*-- Copyright (c) 2014 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved. --*/
/*
Model         	FAW 
Subject Area  	FAW 
Version			FAW 5.3
Name          	ddcf_time_disaggregation: Script to disaggregate the forecast values to DAY level.
			    Disaggregation procedure: 1 - Use Exponential smoothing for disaggregation
									      2 - Use Equal Split for disaggregation
Created by 		SAS R & D Pune, India.
Created on 		11/10/2015 
Last updated 	11/10/2015 

* Copyright (c) 2014 by SAS Institute Inc., Cary, NC 27513

Input Parameters 

History:

*/

%macro ddcf_time_disaggregation(input_tab = , 
								output_tab = ,
								from_interval = ,
								plan_kpi_aggregation_method = ,
								by_var = ,
								target_var = ,
								id_var =, 
								allow_negative_ind =
								);


/* Note: Change the value of disaggregation_procedure as per customer's requirement */


%let disaggregation_procedure = 3;

%let job_rc = 0;
%let err_status_ind =0;
%let job_rc_msg = ;
%global time_disagg_err_ind ;

%let time_disagg_err_ind  = 0;

%global negativeval_ts;

%let negativeval_ts = 0;

%if %sysfunc(exist(&input_tab.)) = 0 %then %do;
	%put NOTE: &input_tab. table does not exist.;
	%goto exit;
%end;

proc sql;
	select count(*) into: input_tab_cnt from &input_tab.;
quit;

%if &input_tab_cnt. = 0 %then %do;
	%put NOTE: &input_tab. table is empty.;
	%goto exit;
%end;

data _null_;
	by_var_cnt = countw("&by_var.");
	call symputx('by_var_cnt',by_var_cnt);
run;


/*====================================================================================================*/
/* START: Exponential smoothing */
/*====================================================================================================*/

%if &disaggregation_procedure. eq 1 and 
	( "%sysfunc(kupcase(&plan_kpi_aggregation_method.))" = "TOTAL" or   
	  "%sysfunc(kupcase(&plan_kpi_aggregation_method.))" = "AVERAGE") 
%then %do; /* START: If disaggregation_procedure eq 1 */

	%ddcf_exp_smoothing_disagg( plan_kpi_aggregation_method = &plan_kpi_aggregation_method. );

%end;  

/*====================================================================================================*/
/* END: Exponential smoothing for aggregation method = Total or Average */
/*====================================================================================================*/

/*====================================================================================================*/
/* START: Equal split */
/*====================================================================================================*/

%else %if &disaggregation_procedure. eq 2 and 
		  ( "%sysfunc(kupcase(&plan_kpi_aggregation_method.))" = "TOTAL" or   
	 	 	"%sysfunc(kupcase(&plan_kpi_aggregation_method.))" = "AVERAGE" )

%then %do; /* START: If disaggregation_procedure eq 2*/

	data &output_tab.(drop= &id_var. &target_var. days_in_interval date_loop rename=(new_date=&id_var. new_target_var= &target_var. ));
		set &input_tab.;
		format new_date date9.;
		&id_var. = INTNX("&from_interval",&id_var.,0,"B");
		days_in_interval= INTNX("&from_interval",&id_var.,0,"E") - INTNX("&from_interval",&id_var.,0,"B")+1;
		do date_loop = 0 to (days_in_interval-1);
			new_date = &id_var. + date_loop;
			%if "%sysfunc(kupcase(&plan_kpi_aggregation_method.))" = "TOTAL" %then %do;
				new_target_var = &target_var./days_in_interval;
			%end;
			%else %do;
				new_target_var = &target_var.;
			%end;
			output;
		end;

	run;
	
	%rcset(&syserr);
	%if (&job_rc>4) %then %do;
		%put ERROR: An error occurred while time level disaggregation;
		%goto exit;
	%end;
	 
%end; /* END: If disaggregation_procedure eq 3*/

/*====================================================================================================*/
/* END: Equal split */
/*====================================================================================================*/


/* %else %if <Condition> %then %do;
	%end;
*/

%else %do;

	%ddcf_exp_smoothing_disagg( plan_kpi_aggregation_method = TOTAL );

%end;

/*Validate that output table is exist */
%if %sysfunc(exist(&output_tab.)) = 0 %then %do;
	%put ERROR: While disaggregating forecast values.;
	%let time_disagg_err_ind = 1;
	%goto exit;
%end;

%exit:

%mend;
%macro ddcf_exp_smoothing_disagg(plan_kpi_aggregation_method =) ;


%let disagg_tmp1_cnt = 0;
/* Subsetting timeseries for which only one record exist for BY group values */
proc sql noprint;
	create table sigleobs_tmp1 as
	select %do i = 1 %to &by_var_cnt.;
				%let rk_string = %sysfunc(kscan(%bquote(&by_var.),&i.));
				&rk_string. ,
		   %end; 
		   count(*) as cnt from &input_tab. 
	group by %do i = 1 %to &by_var_cnt.;
				%let rk_string = %sysfunc(kscan(%bquote(&by_var.),&i.));
				%if &i. eq &by_var_cnt. %then %do;
					&rk_string. 
				%end;
				%else %do;
					&rk_string.,
				%end;
		     %end; 
			 
	having calculated cnt = 1 
	;
quit;

%rcset(&sqlrc);
%if (&job_rc>4) %then %do;
	%put ERROR: An error occurred while time level disaggregation;
	%goto exit;
%end;

%let single_obs_ts = &sqlobs.;

/* equal split option will be used for disaggregation for by variables values that are having only one record. */
%if &single_obs_ts. > 0 %then %do; /* START: If some timeseries having only one record*/
	proc sort data= &input_tab.  out= input_tab_sorted threads;
		by &by_var.; 
	run;

	%rcset(&syserr);
	%if (&job_rc>4) %then %do;
		%put ERROR: An error occurred while time level disaggregation;
		%goto exit;
	%end;

	data sigleobs_tmp2 disagg_tmp1;
		merge input_tab_sorted( in = a) sigleobs_tmp1 (in = b drop=cnt);
		by &by_var.;
		if a=1 and  b=1 then output sigleobs_tmp2;
		else if  a=1 and b=0 then output disagg_tmp1;
	run;

	%rcset(&syserr);
	%if (&job_rc>4) %then %do;
		%put ERROR: An error occurred while time level disaggregation;
		%goto exit;
	%end;

	proc sql noprint;
		select count(*) into: disagg_tmp1_cnt from work.disagg_tmp1;
	quit;
	
	data sigleobs_tmp3(drop= &id_var. &target_var. days_in_interval date_loop rename=(new_date=&id_var. new_target_var= &target_var. ));
		set sigleobs_tmp2;
		format new_date date9.;
		&id_var. = INTNX("&from_interval",&id_var.,0,"B");
		days_in_interval= INTNX("&from_interval",&id_var.,0,"E") - INTNX("&from_interval",&id_var.,0,"B")+1;
		do date_loop = 0 to (days_in_interval-1);
			new_date = &id_var. + date_loop;
			%if "%sysfunc(kupcase(&plan_kpi_aggregation_method.))" = "TOTAL" %then %do;
				new_target_var = &target_var./days_in_interval;
			%end;
			%else %do;
				new_target_var = &target_var.;
			%end;
			output;
		end;

	run;

	%rcset(&syserr);
	%if (&job_rc>4) %then %do;
		%put ERROR: An error occurred while time level disaggregation;
		%goto exit;
	%end;

	
%end; /* END: If some timeseries having only one record*/

%if &single_obs_ts. = 0  or (&single_obs_ts. > 0 and  &disagg_tmp1_cnt. > 0) %then %do;
	proc expand 
		data = %if &single_obs_ts. > 0 %then %do;
					work.disagg_tmp1
				%end;
				%else %do;
					&input_tab.
				%end;
				
        out = disagg_tmp2
        from = &from_interval. 
		to = DAY;

	    by &by_var. ;
        convert &target_var. / observed=&plan_kpi_aggregation_method.;
        id &id_var.;
    run;

	%rcset(&syserr);
	%if (&job_rc>4) %then %do;
		%put ERROR: An error occurred while time level disaggregation;
		%goto exit;
	%end;

	%if %sysfunc(exist(work.sigleobs_tmp3)) %then %do;
		proc append base = disagg_tmp2 data = sigleobs_tmp3 force nowarn;
		run;

		%rcset(&syserr);
		%if (&job_rc>4) %then %do;
			%put ERROR: An error occurred while time level disaggregation;
			%goto exit;
		%end;
	%end;
%end;
%else %do;
	proc datasets nolist nowarn;
		change sigleobs_tmp3 = disagg_tmp2;
	run;
	quit;
	
%end;
/* check for negative values */

proc sql noprint;
	create table negativeval_tmp1 as
	select *,INTNX("&from_interval",&id_var.,0,"B") format = date9. as begining_date 
	from disagg_tmp2 
	where &target_var. < 0
	order by %do i = 1 %to &by_var_cnt.;
				%let rk_string = %sysfunc(kscan(%bquote(&by_var.),&i.));
				&rk_string. ,
		     %end;  
			 calculated begining_date
	;
quit;

%rcset(&sqlrc);
%if (&job_rc>4) %then %do;
	%put ERROR: An error occurred while time level disaggregation;
	%goto exit;
%end;

proc sql noprint;
	create table negativeval_tmp2 as 
	select distinct * from negativeval_tmp1(drop=&id_var. &target_var.);
quit;

%rcset(&sqlrc);
%if (&job_rc>4) %then %do;
	%put ERROR: An error occurred while time level disaggregation;
	%goto exit;
%end;

%let negativeval_ts = &sqlobs.;

%if &negativeval_ts. > 0 and &allow_negative_ind. = 0 %then %do; /* START: If negative timesies are present*/
	
	data negativeval_tmp3 ;
		merge &input_tab.( in = a) negativeval_tmp2 (in = b rename=( begining_date = &id_var. ));
		by &by_var. &id_var. ;
		if a=1 and  b=1 then output negativeval_tmp3;
	run;

	%rcset(&syserr);
	%if (&job_rc>4) %then %do;
		%put ERROR: An error occurred while time level disaggregation;
		%goto exit;
	%end;
	
	data negativeval_tmp4(drop= &id_var. &target_var. days_in_interval date_loop rename=(new_date=&id_var. new_target_var= &target_var. ));
		set negativeval_tmp3;
		format new_date date9.;
		&id_var. = INTNX("&from_interval",&id_var.,0,"B");
		days_in_interval= INTNX("&from_interval",&id_var.,0,"E") - INTNX("&from_interval",&id_var.,0,"B")+1;
		do date_loop = 0 to (days_in_interval-1);
			new_date = &id_var. + date_loop;
			new_target_var = &target_var./days_in_interval;
			output;
		end;

	run;

	%rcset(&syserr);
	%if (&job_rc>4) %then %do;
		%put ERROR: An error occurred while time level disaggregation;
		%goto exit;
	%end;

	
	proc sort data = work.disagg_tmp2 ;
		by &by_var. &id_var.;
	run;

	data disagg_tmp3;
		merge disagg_tmp2(in=a) negativeval_tmp4(in=b keep= &id_var. &target_var. &by_var. );
		by  &by_var. &id_var. ;
	run;

	%rcset(&syserr);
	%if (&job_rc>4) %then %do;
		%put ERROR: An error occurred while time level disaggregation;
		%goto exit;
	%end;
	
%end; /* END: If negative timeseries are present*/



%if &negativeval_ts. > 0 and &allow_negative_ind. = 0 %then %do;
	data &output_tab.;
	set disagg_tmp3;
	run; 
%end;
%else %do;
	data &output_tab.;
	set disagg_tmp2;
	run;
%end;

%exit:

%mend;

